####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                           SET PATHS                                    ####
path_root         <- 'N:/UDMTD/UDTMDCOM/Cursos.Seminarios/BIGSURV20/edj-paper/supplMaterial'
#path_root         <- 'N:/UDTMDCOM/Cursos.Seminarios/BIGSURV20/edj-paper/supplMaterial'
path_source       <- file.path(path_root, 'source')
path_input        <- file.path(path_root, 'simulatorInputData')
path_output       <- file.path(path_root, 'simulatorOutputData')
path_intermediate <- file.path(path_root, 'intermediateData')
path_image        <- file.path(path_root, 'image')

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                  LOAD PACKAGES AND FUNCTIONS                           ####
library(xml2)
library(data.table)
library(stringr)
library(ggplot2)
library(ggimage)
library(scales)
library(raster)
library(tidyr)                # transform xml documents to tables (tibbles)
library(rgeos)                # spatial data
library(raster)               # spatial data
library(sf)                   # spatial data
library(gganimate)
library(magick)
library(animation)

# Function to read the input files of the simulator
source(file.path(path_source, 'read_inputSimulator.R'))

# Function tileEquivalence to compute the equivalence between 
#          rastercell (R package raster) and tiles (simulator)
source(file.path(path_source, 'tileEquivalence.R'))

# Function to read the output files of the simulator
source(file.path(path_source, 'read_outputSimulator.R'))

# Function to create the emissions matrix
source(file.path(path_source, 'build_emissionsMatrix.R'))

# Function to create the data.table with the cetroids and the regions
source(file.path(path_source, 'build_centroid_regions.R'))

# Function to create a raster objecto from a map WKT and grid params.
source(file.path(path_source, 'createRaster.R'))

# Function to calculate de density for a variable in a simulation.
source(file.path(path_source, 'simulation_density.R'))




####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                  READ INPUT DATA FILES                                       ####
result_input  <- read_inputSimulator(path_input)

simulation.xml               <- result_input$simulation.xml
antennaQualitySignalParam.dt <- result_input$antennas_xml
map                          <- result_input$map
persons.xml                  <- result_input$persons.xml
probabilities.xml            <- result_input$probabilities.xml
fileProbabilitiesName        <- result_input$filesNames$fileProbabilitiesName
fileGridName                 <- result_input$filesNames$fileGridName
filePersonsName              <- result_input$filesNames$filePersonsName
fileAntennasInfoName         <- result_input$filesNames$fileAntennasInfoName
fileSignalName               <- result_input$filesNames$fileSignalName
fileCoverName                <- result_input$filesNames$fileCoverName
MNOName                      <- result_input$inputParameters$MNOName
probabilitiesName            <- result_input$inputParameters$probabilitiesName
connectionType               <- result_input$inputParameters$connectionType
conn_threshold               <- result_input$inputParameters$conn_threshold
prob_secDev                  <- result_input$inputParameters$prob_secDev
initialTime                  <- result_input$inputParameters$initialTime
finalTime                    <- result_input$inputParameters$finalTime
t_increment                  <- result_input$inputParameters$t_increment
times                        <- result_input$inputParameters$times
num_persons                  <- result_input$inputParameters$num_persons
min_age                      <- result_input$inputParameters$min_age
max_age                      <- result_input$inputParameters$max_age
speed_walk                   <- result_input$inputParameters$speed_walk
speed_car                    <- result_input$inputParameters$speed_car
percent_home                 <- result_input$inputParameters$percent_home


####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                  READ OUTPUT DATA FILES                                ####
result_output <- read_outputSimulator(path_output, path_intermediate,
                                      fileGridName, filePersonsName,
                                      fileAntennasInfoName, fileSignalName,
                                      fileCoverName, 
                                      map, antennaQualitySignalParam.dt,
                                      connectionType, conn_threshold)


antenna.dt              <- result_output$antenna
events.dt               <- result_output$events
persons_parsed.dt       <- result_output$persons_parsed
gridParam               <- result_output$gridParam
RSS_Sim.dt              <- result_output$RSS_Sim
coverArea               <- result_output$coverArea
coverArea_sf            <- result_output$coverArea_sf
positionsConnections.dt <- result_output$positionsConnections
anteAreas.dt            <- result_output$anteAreas
tileEquiv.dt            <- result_output$tileEquiv
centroidCoord.dt        <- result_output$centroidCoord
ncol_grid               <- result_output$outputParameters$ncol_grid
nrow_grid               <- result_output$outputParameters$nrow_grid
tile_sizeX              <- result_output$outputParameters$tile_sizeX
tile_sizeY              <- result_output$outputParameters$tile_sizeY
ntiles_x                <- result_output$outputParameters$ntiles_x
ntiles_y                <- result_output$outputParameters$ntiles_y
ntiles                  <- result_output$outputParameters$ntiles



####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
#####                           SAVE RDS FILES                              #####

cat('Saving RDS Files...\n')
cat('     Saving RData: filesNames.RData, inputParameters.RData ...')
save(fileProbabilitiesName, fileGridName, 
     filePersonsName, fileAntennasInfoName, fileSignalName, fileCoverName,
     file = file.path(path_intermediate, "filesNames.RData"))

save(MNOName, probabilitiesName, connectionType, conn_threshold, prob_secDev,
     initialTime, finalTime, t_increment, times, 
     num_persons, min_age, max_age, speed_walk, speed_car, percent_home,
     file = file.path(path_intermediate, "inputParameters.RData"))
cat('ok.\n')

cat('     Saving RDS: simulation.xml.rds, antennaQualitySignalParam.dt.rds, map.rds ...')
saveRDS(simulation.xml, file.path(path_intermediate, "simulation.xml.rds"))
saveRDS(antennaQualitySignalParam.dt, file.path(path_intermediate, "antennaQualitySignalParam.dt.rds"))
saveRDS(map, file.path(path_intermediate, "map.rds"))
cat('ok.\n')


cat('     Saving RData: outputParameters.RData ...')
save(ncol_grid, nrow_grid, tile_sizeX, tile_sizeY, ntiles_x, ntiles_y, ntiles,
     file = file.path(path_intermediate, "outputParameters.RData"))
cat('ok.\n')

cat('     Saving RDS: antenna.dt.rds, \nevents.dt.rds, \npersons_parsed.dt.rds, \ngridParam.rds, \nRSS_Sim.dt.rds, \ncoverArea.rds, \ncoverArea_sf.rds, \npositionsConnections.dt.rds, \nanteAreas.dt.rds ...')
saveRDS(antenna.dt, file.path(path_intermediate, "antenna.dt.rds"))
saveRDS(events.dt, file.path(path_intermediate, "events.dt.rds"))
saveRDS(persons_parsed.dt, file.path(path_intermediate, "persons_parsed.dt.rds"))
saveRDS(gridParam, file.path(path_intermediate, "gridParam.rds"))
saveRDS(RSS_Sim.dt, file.path(path_intermediate, "RSS_Sim.dt.rds"))
saveRDS(coverArea, file.path(path_intermediate, "coverArea.rds"))
saveRDS(coverArea_sf, file.path(path_intermediate, "coverArea_sf.rds"))
saveRDS(positionsConnections.dt, file.path(path_intermediate, "positionsConnections.dt.rds"))
saveRDS(anteAreas.dt, file.path(path_intermediate, "anteAreas.dt.rds"))
saveRDS(tileEquiv.dt, file.path(path_intermediate, "tileEquiv.dt.rds"))

cat('ok.\n')

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                  BUILD AND SAVE CENTROIDS REGIONS                               ####

centroidCoord.dt <- build_centroid_regions(ntiles_x, ntiles_y, tile_sizeX, tile_sizeY,
                                                 ncol_grid, nrow_grid)
saveRDS(centroidCoord.dt, file.path(path_intermediate, "centroidCoord.dt.rds"))

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                  READ RDS AND SP DATA FILES                            ####
individuals.dt <- readRDS(file.path(path_intermediate, 'persons_parsed.dt.rds'))[
  , hasDevice := ifelse(is.na(deviceID_1), FALSE, TRUE)][
  , nDevices  := ifelse(is.na(deviceID_1) & is.na(deviceID_2), 0,
                        ifelse(!is.na(deviceID_1) & is.na(deviceID_2), 1, 2))]

map.sp         <- readRDS(file.path(path_intermediate, 'map.rds'))

gridParam      <- readRDS(file.path(path_intermediate, 'gridParam.rds'))
ncols <- gridParam[['No Tiles Y']] # está ya en los parámetros: ncol_grid
nrows <- gridParam[['No Tiles X']] # está ya en los parámetros: nrow_grid

RSS.dt         <- readRDS(file.path(path_intermediate, 'RSS_Sim.dt.rds'))

antennas.dt    <- readRDS(file.path(path_intermediate, 'antenna.dt.rds'))[
  , icon := file.path(path_image, 'omnidirectional-antenna-tiny.png')]

coverArea.list <- readRDS(file.path(path_intermediate, 'coverArea.rds'))

events.dt <- readRDS(file.path(path_intermediate, 'events.dt.rds'))
  
####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                  CREATE EMISSION MATRIX                                ####

eventLoc_rasterCell.matrix <- build_emissionsMatrix(RSS.dt, connectionType)
saveRDS(eventLoc_rasterCell.matrix, file.path(path_intermediate, "eventLoc_rasterCell.matrix.rds"))


####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                  PLOT INITIAL POSITION OF INDIVIDUALS                  ####
p <- ggplot() + 
  theme(panel.background = element_blank(), 
        panel.grid.major = element_line(color = 'grey100'), panel.grid.minor = element_line(color = 'grey75'),
        axis.title.x = element_blank(), axis.title.y = element_blank(),
        axis.ticks.x = element_blank(), axis.ticks.y = element_blank(),
        axis.text.x = element_blank(),  axis.text.y = element_blank(),
        axis.line = element_line(colour = "white"),
        plot.margin=unit(c(0, 0, 0, 0), "cm"),
        legend.position = 'top', legend.title=element_blank()) +
  geom_polygon(
    aes(x = map.sp@polygons[[1]]@Polygons[[1]]@coords[, 'x'], 
        y = map.sp@polygons[[1]]@Polygons[[1]]@coords[, 'y']), 
        fill = "grey75", alpha = 0.01) +
  geom_image(data = antennas.dt, aes(x = x, y = y, image = icon), size = 0.03, color = 'black') +
  geom_point(data = individuals.dt[time == 0], aes(x = x, y = y, color = hasDevice), size = 1) +
  coord_fixed() +
  scale_color_manual(values = c('red', 'blue'), labels = c('Without device', 'With device')) +
  scale_y_continuous(minor_breaks = seq(0, 1e4, 250), breaks = seq(0, 1e4, 250)) +
  scale_x_continuous(minor_breaks = seq(0, 1e4, 250), breaks = seq(0, 1e4, 250))
pdf(file = file.path(path_image, 'initialPositions.pdf'))
p
dev.off()



####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                      PLOT ANIMATION OF INDIVIDUALS                     ####
p <- ggplot() + 
  theme(panel.background = element_blank(), 
        panel.grid.major = element_line(color = 'grey100'), panel.grid.minor = element_line(color = 'grey75'),
        axis.title.x = element_blank(), axis.title.y = element_blank(),
        axis.ticks.x = element_blank(), axis.ticks.y = element_blank(),
        axis.text.x = element_blank(),  axis.text.y = element_blank(),
        axis.line = element_line(colour = "white"),
        plot.margin=unit(c(0, 0, 0, 0), "cm"),
        legend.position = 'top', legend.title=element_blank()) +
  geom_polygon(
    aes(x = map.sp@polygons[[1]]@Polygons[[1]]@coords[, 'x'], 
        y = map.sp@polygons[[1]]@Polygons[[1]]@coords[, 'y']), 
    fill = "#7D7D7D", alpha = 0.1) +
  geom_image(data = antennas.dt, aes(x = x, y = y, image = icon), size = 0.03) +
  geom_point(data = individuals.dt[!is.na(time)], aes(x = x, y = y, color = hasDevice), size = 1) +
  coord_fixed() +
  scale_color_manual(values = c('red', 'blue'), labels = c('Without device', 'With device')) +
  scale_y_continuous(minor_breaks = seq(0, 1e4, 250), breaks = seq(0, 1e4, 250)) +
  scale_x_continuous(minor_breaks = seq(0, 1e4, 250), breaks = seq(0, 1e4, 250)) +
  transition_time(time) +
  labs(title = "Time: {frame_time} s")
anim <- animate(p, fps = 10)
magick::image_write(anim, path= file.path(path_image, "individuals.gif"))
antennas.dt[, icon := NULL] # icons not needed anymore

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                  PLOT RASTER FOR RSS FOR AN ANTENNA                    ####
antID <- '03'
RSS.raster <- raster(
  ncol = ncols, nrow = nrows,
  xmn = gridParam[['Origin X']], ymn = gridParam[['Origin Y']],
  xmx = gridParam[['Origin X']] + nrows * gridParam[['X Tile Dim']],
  ymx = gridParam[['Origin Y']] + nrows * gridParam[['Y Tile Dim']])
rasterValues.dt <- merge(RSS.dt[antennaID == antID][, .(tile, RSS_ori)], tileEquivalence(nrows, ncols), by.x = 'tile', by.y = 'tileID', all = TRUE)[
  , tile := NULL][
    order(rasterCellID)]
values(RSS.raster) <- rasterValues.dt$RSS_ori

pdf(file = file.path(path_image, 'RSS_raster.pdf'))
plot(RSS.raster, legend = FALSE)
plot(map.sp, add = TRUE)
dev.off()

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                  PLOT RASTER FOR SDM FOR AN ANTENNA                    ####
SDM.raster <- raster(
  ncol = ncols, nrow = nrows,
  xmn = gridParam[['Origin X']], ymn = gridParam[['Origin Y']],
  xmx = gridParam[['Origin X']] + nrows * gridParam[['X Tile Dim']],
  ymx = gridParam[['Origin Y']] + nrows * gridParam[['Y Tile Dim']])
rasterValues.dt <- merge(RSS.dt[antennaID == antID][, .(tile, SDM_ori)], tileEquivalence(nrows, ncols), by.x = 'tile', by.y = 'tileID', all = TRUE)[
  , tile := NULL][
    order(rasterCellID)]
values(SDM.raster) <- rasterValues.dt$SDM_ori
pdf(file = file.path(path_image, 'SDM_raster.pdf'))
plot(SDM.raster, legend = FALSE)
plot(map.sp, add = TRUE)
dev.off()



####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                     MAIN PARAMETERS OF SCENARIO                        ####
(nIndividuals <- length(unique(individuals.dt$personID)))
(nDevices <- length(unique(
  c(individuals.dt[!is.na(deviceID_1), deviceID_1], 
    individuals.dt[!is.na(deviceID_1) & !is.na(deviceID_2), deviceID_2]))))
(nTiles <- gridParam$`No Tiles X` * gridParam$`No Tiles Y`)
(gridParam$`X Tile Dim`)
(gridParam$`Y Tile Dim`)
(nAntennas <- length(unique(antennas.dt$antennaID)))

(table(antennas.dt$attenuationfactor))
(table(antennas.dt$power))
(table(RSS.dt$nAntCover))
(table(antennas.dt$SSteep))
(table(antennas.dt$Smid))

rbind(
  summary(antennas.dt$attenuationfactor),
  summary(antennas.dt$power),
  summary(antennas.dt$cover_radio),
  summary(antennas.dt$SSteep),
  summary(antennas.dt$Smid))


####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####               MORE DETAIL DESCRIPTION OF SCENARIO                      ####

#### * Density of antennas                                                  ####
output_density <- simulation_density(info.dt = antennas.dt, 
                   map = map.sp, gridParam = gridParam, 
                   variable = "antennaID", 
                   margin = 3)

allCells.dt <- output_density$allCells.dt
map.raster  <- output_density$map.raster

values(map.raster) <- allCells.dt[, total]
# values(map.raster) <- allCells.dt[, density_tiles]
# values(map.raster) <- allCells.dt[, density_area]
plot(map.raster)


#### * Density of individuals                                               ####
times <- sort(unique(individuals.dt$time))
individualDensityLayers.list <- vector(mode = 'list', length = length(times))
names(individualDensityLayers.list) <- as.character(times)
for (locTime in times){
  cat(paste0('Layer for time ', locTime, '... '))
  output_density     <- simulation_density(
    info.dt = individuals.dt[time == locTime], 
    map = map.sp, gridParam = gridParam, 
    variable = "personID", margin = 2)
  allCells.dt        <- output_density$allCells.dt
  map.raster         <- output_density$map.raster
  values(map.raster) <- allCells.dt[, density_tiles]
  individualDensityLayers.list[[as.character(locTime)]] <- map.raster
  cat(' ok.\n')
}
saveRDS(individualDensityLayers.list, file.path(path_intermediate, 'individualDensityLayers.list.rds'))
individualDensity.brick <- brick(Reduce(stack, individualDensityLayers.list))
animation::saveGIF(
  raster::animate(individualDensity.brick, n = 1, pause = 0.01, main = ''), 
  movie.name = file.path(path_image, 'individualDensity.gif'))

#### * Density of devices                                                   ####
times <- sort(unique(events.dt$time))
deviceDensityLayers.list <- vector(mode = 'list', length = length(times))
names(deviceDensityLayers.list) <- as.character(times)
for (locTime in times){
  cat(paste0('Layer for time ', locTime, '... '))
  output_density     <- simulation_density(
    info.dt = events.dt[time == locTime], 
    map = map.sp, gridParam = gridParam, 
    variable = "deviceID", margin = 2)
  allCells.dt        <- output_density$allCells.dt
  map.raster         <- output_density$map.raster
  values(map.raster) <- allCells.dt[, density_tiles]
  deviceDensityLayers.list[[as.character(locTime)]] <- map.raster
  cat(' ok.\n')
}
saveRDS(deviceDensityLayers.list, file.path(path_intermediate, 'deviceDensityLayers.list.rds'))
deviceDensity.brick <- brick(Reduce(stack, deviceDensityLayers.list))
animation::saveGIF(
  raster::animate(deviceDensity.brick, n = 1, pause = 0.01, main = ''), 
  movie.name = file.path(path_image, 'deviceDensity.gif'))


#### * Coarsed-grained density of events per antenna                        ####
times <- sort(unique(events.dt$time))
events_perAntenna.dt <- copy(events.dt)[
  , .(time, deviceID, antennaID)][
    antennas.dt[, .(antennaID, rasterCell)], on = "antennaID"]
events_perAntenna.list <- vector(mode = 'list', length = length(times))
names(events_perAntenna.list) <- as.character(times)
for (locTime in times){
  cat(paste0('Layer for time ', locTime, '... '))
  output_density     <- simulation_density(
    info.dt = events_perAntenna.dt[time == locTime], 
    map = map.sp, gridParam = gridParam, 
    variable = "deviceID", margin = 2)
  allCells.dt        <- output_density$allCells.dt
  map.raster         <- output_density$map.raster
  values(map.raster) <- allCells.dt[, density_tiles]
  events_perAntenna.list[[as.character(locTime)]] <- map.raster
  cat(' ok.\n')
}
saveRDS(events_perAntenna.list, file.path(path_intermediate, 'events_perAntenna.list.rds'))
events_perAntenna.brick <- brick(Reduce(stack, events_perAntenna.list))
animation::saveGIF(
  raster::animate(events_perAntenna.brick, n = 1), 
  movie.name = file.path(path_image, 'events_perAntenna.gif'))

