####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                           SET PATHS                                    ####
path_root         <- 'N:/UDMTD/UDTMDCOM/Cursos.Seminarios/BIGSURV20/edj-paper/supplMaterial'
#path_root         <- 'N:/UDTMDCOM/Cursos.Seminarios/BIGSURV20/edj-paper/supplMaterial'
path_source       <- file.path(path_root, 'source')
path_input        <- file.path(path_root, 'simulatorInputData')
path_output       <- file.path(path_root, 'simulatorOutputData')
path_intermediate <- file.path(path_root, 'intermediateData')
path_image        <- file.path(path_root, 'image')

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                  LOAD PACKAGES AND FUNCTIONS                           ####
library(xml2)
library(data.table)
library(destim)
library(stringr)
library(Matrix)
library(ggplot2)
library(ggimage)
library(scales)
library(raster)
library(tidyr)                # transform xml documents to tables (tibbles)
library(rgeos)                # spatial data
library(raster)               # spatial data
library(sf)                   # spatial data
library(gganimate)
library(magick)

# Function to create rasters
source(file.path(path_source, 'createRaster.R'))

# Function to construct the transition matrix from an HMM object
source(file.path(path_source, 'build_transMatrix.R'))


# Function to compute event location (likelihoods)
source(file.path(path_source, 'computeEventLoc.R'))

# Function to fit an HMM and compute posterior location probabilities
source(file.path(path_source, 'compute_HMM.R'))

# Function to transform de output of compute_HMM
source(file.path(path_source, 'transform_postLoc.R'))


####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                      READ RDS DATA FILES                               ####
RSS.dt <- readRDS(file.path(path_intermediate, 'RSS.dt.rds'))

gridParam      <- readRDS(file.path(path_intermediate, 'gridParam.rds'))
ncols <- gridParam[['No Tiles Y']] # está ya en los parámetros: ncol_grid
nrows <- gridParam[['No Tiles X']] # está ya en los parámetros: nrow_grid
nTiles <- ncols * nrows

simulationParam <- readRDS(file.path(path_intermediate, 'simulation.xml.rds'))
initialTime <- as.numeric(simulationParam$simulation$start_time[[1]])
timeIncrem  <- as.numeric(simulationParam$simulation$time_increment[[1]])
finalTime   <- as.numeric(simulationParam$simulation$end_time[[1]]) - timeIncrem

events.dt <- readRDS(file.path(path_intermediate, 'events.dt.rds'))

tileEquiv.dt <- readRDS(file.path(path_intermediate, 'tileEquiv.dt.rds'))

map.sp <- readRDS(file.path(path_intermediate, 'map.rds'))

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                INITIAL STATE DISTRIBUTION (PRIOR)                      ####
nTiles <- length(unique(tileEquiv.dt$tile))
initialDistr_RSS_uniform.vec <- initialDistr_SDM_uniform.vec <- rep(1 / nTiles, nTiles)

initialDistr_RSS_network.dt <- RSS.dt[
  , watt := 10**( (RSS - 30) / 10 )][
  , total := sum(watt, na.rm = TRUE)][
  , list(num = sum(watt, na.rm = TRUE), total = unique(total)), by = 'rasterCell'][
  , prior_network := num / total][
  order(rasterCell)]
initialDistr_RSS_network.vec <- initialDistr_RSS_network.dt$prior_network

initialDistr_SDM_network.dt <- RSS.dt[
  , total := sum(SDM_ori, na.rm = TRUE)][
  , list(num = sum(SDM_ori, na.rm = TRUE), total = unique(total)), by = 'rasterCell'][
  , prior_network := num / total][
  order(rasterCell)]
initialDistr_SDM_network.vec <- initialDistr_SDM_network.dt$prior_network


####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                COMPUTE EMISSION PROB MATRICES                          ####
emissionProb_RSS.mat <- computeEventLoc(RSS.dt, 'RSS', 'RSS', by.grid = 'rasterCell')
pdf(file = file.path(path_image, 'emissionProb_RSS.pdf'))
plot(raster(emissionProb_RSS.mat), legend = FALSE)
dev.off()

emissionProb_SDM.mat <- computeEventLoc(RSS.dt, 'SDM', 'SDM_ori', by.grid = 'rasterCell')
pdf(file = file.path(path_image, 'emissionProb_SDM.pdf'))
plot(raster(emissionProb_SDM.mat), legend = FALSE)
dev.off()


####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                          HMM INITIALIZATION                            ####
model_RSS_uniform <- model_SDM_uniform <- model_RSS_network <- model_SDM_network <-HMMrectangle(nrows, ncols)

emissions(model_RSS_uniform) <- emissions(model_RSS_network) <- emissionProb_RSS.mat
emissions(model_SDM_uniform) <- emissions(model_SDM_network) <- emissionProb_SDM.mat

model_RSS_uniform <- initparams(model_RSS_uniform)
model_SDM_uniform <- initparams(model_SDM_uniform)
model_RSS_network <- initparams(model_RSS_network)
model_SDM_network <- initparams(model_SDM_network)

model_RSS_uniform <- minparams(model_RSS_uniform)
model_SDM_uniform <- minparams(model_SDM_uniform)
model_RSS_network <- minparams(model_RSS_network)
model_SDM_network <- minparams(model_SDM_network)


istates(model_RSS_uniform) <- initialDistr_RSS_uniform.vec
istates(model_SDM_uniform) <- initialDistr_SDM_uniform.vec
istates(model_RSS_network) <- initialDistr_RSS_network.vec
istates(model_SDM_network) <- initialDistr_SDM_network.vec


####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                          TIME PADDING                                  ####
vMax_kmh <- 150
vMax_ms <- vMax_kmh * 1000 / 3600
distMax <- vMax_ms * timeIncrem
pad_coef <- as.integer(ceiling(distMax / max(gridParam$`X Tile Dim`, gridParam$`Y Tile Dim`)))
pad_coef <- pad_coef + 1 

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####              HMM FITTING AND POSTLOC COMPUTATION                       ####
model_RSS_uniform.list      <- list()
model_SDM_uniform.list      <- list()
model_RSS_network.list      <- list()
model_SDM_network.list      <- list()
postLocProb_RSS_uniform.list      <- list()
postLocProb_SDM_uniform.list      <- list()
postLocProb_RSS_network.list      <- list()
postLocProb_SDM_network.list      <- list()
postJointLocProb_RSS_uniform.list <- list()
postJointLocProb_SDM_uniform.list <- list()
postJointLocProb_RSS_network.list <- list()
postJointLocProb_SDM_network.list <- list()

times.dt <- data.table(time = seq(from = initialTime, to = finalTime, by = timeIncrem))
times <- times.dt$time
deviceIDs <- sort(unique(events.dt$deviceID))
for(idx_device in seq(along = deviceIDs)){
  
  devID <- deviceIDs[idx_device]
  cat(paste0('    device ', devID,'...\n'))
  
  # Selecting device events (observed variables)
  cat('       selecting events...')
  events_deviceID.dt <- events.dt[
    deviceID == devID]
  events_deviceID.dt <- merge(events_deviceID.dt, times.dt, by = 'time', all = TRUE)[
    , deviceID := devID]
  antennas_deviceID <- unlist(events_deviceID.dt[, c("antennaID")])
  cat(' ok.\n')  
  
  if(!all(is.na(antennas_deviceID))){
    
    cat(paste0(idx_device, ' of ', length(deviceIDs), '.\n'))  
    
    #####                Fit and compute HMM model                 ####
    cat('       fit and compute HMM model...\n')
    cat('         RSS and uniform initial state...\n')
    HMMmodel_output_RSS_uniform <- compute_HMM(
      model = model_RSS_uniform, observedValues = antennas_deviceID, 
      pad_coef = pad_coef, init = TRUE)
    
    model_RSS_uniform.list[[as.character(devID)]] <- HMMmodel_output_RSS_uniform$model_devID
    postLocProb_HMM_RSS_uniform_deviceID.matrix <- HMMmodel_output_RSS_uniform$postLocP 
    postJointLocProb_HMM_RSS_uniform_deviceID.matrix <- HMMmodel_output_RSS_uniform$postJointLocP
    
    transfom_output <- transform_postLoc(
      postLocP = postLocProb_HMM_RSS_uniform_deviceID.matrix,
      postJointLocP = postJointLocProb_HMM_RSS_uniform_deviceID.matrix,
      observedValues = antennas_deviceID,
      times = times, t_increment = timeIncrem, 
      ntiles = nTiles, pad_coef = pad_coef,
      tileEquiv.dt = tileEquiv.dt, devID = devID,
      sparse_postLocP.dt = FALSE, sparse_postJointLocP.dt = TRUE)
    
    postLocProb_HMM_RSS_uniform_deviceID.dt <- transfom_output$postLocProb_HMM_deviceID.dt[
      eventLoc > 0][
      eventLoc < 0, eventLoc := 0][
      , eventLoc := eventLoc / sum(eventLoc), by = c('time')]
    postLocProb_RSS_uniform.list[[as.character(devID)]] <- postLocProb_HMM_RSS_uniform_deviceID.dt[
      , .(time, tile, eventLoc, device)]
    
    postJointLocProb_HMM_RSS_uniform_deviceID.dt <- transfom_output$postJointLocProb_HMM_deviceID.dt[
      eventLoc > 0][
      eventLoc < 0, eventLoc := 0][
      , eventLoc := eventLoc / sum(eventLoc), by = c('time_from')]
    postJointLocProb_RSS_uniform.list[[as.character(devID)]] <- postJointLocProb_HMM_RSS_uniform_deviceID.dt[
      , .(time_from, time_to, tile_from, tile_to, eventLoc, device)]

    cat('         RSS and network initial state...\n')
    HMMmodel_output_RSS_network <- compute_HMM(
      model = model_RSS_network, observedValues = antennas_deviceID, 
      pad_coef = pad_coef, init = TRUE)
    
    model_RSS_network.list[[as.character(devID)]] <- HMMmodel_output_RSS_network$model_devID
    postLocProb_HMM_RSS_network_deviceID.matrix <- HMMmodel_output_RSS_network$postLocP 
    postJointLocProb_HMM_RSS_network_deviceID.matrix <- HMMmodel_output_RSS_network$postJointLocP
    
    transfom_output <- transform_postLoc(
      postLocP = postLocProb_HMM_RSS_network_deviceID.matrix,
      postJointLocP = postJointLocProb_HMM_RSS_network_deviceID.matrix,
      observedValues = antennas_deviceID,
      times = times, t_increment = timeIncrem, 
      ntiles = nTiles, pad_coef = pad_coef,
      tileEquiv.dt = tileEquiv.dt, devID = devID,
      sparse_postLocP.dt = FALSE, sparse_postJointLocP.dt = TRUE)
    
    postLocProb_HMM_RSS_network_deviceID.dt <- transfom_output$postLocProb_HMM_deviceID.dt[
      eventLoc > 0][
      eventLoc < 0, eventLoc := 0][
        , eventLoc := eventLoc / sum(eventLoc), by = c('time')]
    postLocProb_RSS_network.list[[as.character(devID)]] <- postLocProb_HMM_RSS_network_deviceID.dt[
      , .(time, tile, eventLoc, device)]
    
    postJointLocProb_HMM_RSS_network_deviceID.dt <- transfom_output$postJointLocProb_HMM_deviceID.dt[
      eventLoc > 0][
      eventLoc < 0, eventLoc := 0][
        , eventLoc := eventLoc / sum(eventLoc), by = c('time_from')]

    postJointLocProb_RSS_network.list[[as.character(devID)]] <- postJointLocProb_HMM_RSS_network_deviceID.dt[
      , .(time_from, time_to, tile_from, tile_to, eventLoc, device)]

    cat('       fit and compute HMM model...\n')
    cat('         SDM and uniform initial state...\n')
    HMMmodel_output_SDM_uniform <- compute_HMM(
      model = model_SDM_uniform, observedValues = antennas_deviceID, 
      pad_coef = pad_coef, init = TRUE)
    
    model_SDM_uniform.list[[as.character(devID)]] <- HMMmodel_output_SDM_uniform$model_devID
    postLocProb_HMM_SDM_uniform_deviceID.matrix <- HMMmodel_output_SDM_uniform$postLocP 
    postJointLocProb_HMM_SDM_uniform_deviceID.matrix <- HMMmodel_output_SDM_uniform$postJointLocP
    
    transfom_output <- transform_postLoc(
      postLocP = postLocProb_HMM_SDM_uniform_deviceID.matrix,
      postJointLocP = postJointLocProb_HMM_SDM_uniform_deviceID.matrix,
      observedValues = antennas_deviceID,
      times = times, t_increment = timeIncrem, 
      ntiles = nTiles, pad_coef = pad_coef,
      tileEquiv.dt = tileEquiv.dt, devID = devID,
      sparse_postLocP.dt = FALSE, sparse_postJointLocP.dt = TRUE)
    
    postLocProb_HMM_SDM_uniform_deviceID.dt <- transfom_output$postLocProb_HMM_deviceID.dt[
      eventLoc > 0][
      eventLoc < 0, eventLoc := 0][
      , eventLoc := eventLoc / sum(eventLoc), by = c('time')]
    postLocProb_SDM_uniform.list[[as.character(devID)]] <- postLocProb_HMM_SDM_uniform_deviceID.dt[
      , .(time, tile, eventLoc, device)]
    
    postJointLocProb_HMM_SDM_uniform_deviceID.dt <- transfom_output$postJointLocProb_HMM_deviceID.dt[
      eventLoc > 0][
      eventLoc < 0, eventLoc := 0][
        , eventLoc := eventLoc / sum(eventLoc), by = c('time_from')]
    postJointLocProb_SDM_uniform.list[[as.character(devID)]] <- postJointLocProb_HMM_SDM_uniform_deviceID.dt[
      , .(time_from, time_to, tile_from, tile_to, eventLoc, device)]
    
    cat('         SDM and network initial state...\n')
    HMMmodel_output_SDM_network <- compute_HMM(
      model = model_SDM_network, observedValues = antennas_deviceID, 
      pad_coef = pad_coef, init = TRUE)
    
    model_SDM_network.list[[as.character(devID)]] <- HMMmodel_output_SDM_network$model_devID
    postLocProb_HMM_SDM_network_deviceID.matrix <- HMMmodel_output_SDM_network$postLocP 
    postJointLocProb_HMM_SDM_network_deviceID.matrix <- HMMmodel_output_SDM_network$postJointLocP
    
    transfom_output <- transform_postLoc(
      postLocP = postLocProb_HMM_SDM_network_deviceID.matrix,
      postJointLocP = postJointLocProb_HMM_SDM_network_deviceID.matrix,
      observedValues = antennas_deviceID,
      times = times, t_increment = timeIncrem, 
      ntiles = nTiles, pad_coef = pad_coef,
      tileEquiv.dt = tileEquiv.dt, devID = devID,
      sparse_postLocP.dt = FALSE, sparse_postJointLocP.dt = TRUE)
    
    postLocProb_HMM_SDM_network_deviceID.dt <- transfom_output$postLocProb_HMM_deviceID.dt[
      eventLoc > 0][
      eventLoc < 0, eventLoc := 0][
      , eventLoc := eventLoc / sum(eventLoc), by = c('time')]
    postLocProb_SDM_network.list[[as.character(devID)]] <- postLocProb_HMM_SDM_network_deviceID.dt[
      , .(time, tile, eventLoc, device)]
    
    postJointLocProb_HMM_SDM_network_deviceID.dt <- transfom_output$postJointLocProb_HMM_deviceID.dt[
      eventLoc > 0][
      eventLoc < 0, eventLoc := 0][
        , eventLoc := eventLoc / sum(eventLoc), by = c('time_from')]
    postJointLocProb_SDM_network.list[[as.character(devID)]] <- postJointLocProb_HMM_SDM_network_deviceID.dt[
      , .(time_from, time_to, tile_from, tile_to, eventLoc, device)]
    
    rm(postLocProb_HMM_RSS_uniform_deviceID.matrix, postLocProb_HMM_RSS_network_deviceID.matrix,
       postLocProb_HMM_SDM_uniform_deviceID.matrix, postLocProb_HMM_SDM_network_deviceID.matrix)
    
    rm(postJointLocProb_HMM_RSS_uniform_deviceID.matrix, postJointLocProb_HMM_RSS_network_deviceID.matrix,
       postJointLocProb_HMM_SDM_uniform_deviceID.matrix, postJointLocProb_HMM_SDM_network_deviceID.matrix)
    
    rm(transfom_output)
  } 
}
saveRDS(model_RSS_uniform.list, file.path(path_intermediate, 'model_RSS_uniform.list.rds'))
saveRDS(model_RSS_network.list, file.path(path_intermediate, 'model_RSS_network.list.rds'))
saveRDS(model_SDM_uniform.list, file.path(path_intermediate, 'model_SDM_uniform.list.rds'))
saveRDS(model_SDM_network.list, file.path(path_intermediate, 'model_SDM_network.list.rds'))

postLocProb_RSS_uniform.dt <- rbindlist(postLocProb_RSS_uniform.list)[
  , model := 'RSS'][
  , prior := 'uniform']
postLocProb_RSS_network.dt <- rbindlist(postLocProb_RSS_network.list)[
  , model := 'RSS'][
  , prior := 'network']
postLocProb_SDM_uniform.dt <- rbindlist(postLocProb_SDM_uniform.list)[
  , model := 'SDM'][
  , prior := 'uniform']
postLocProb_SDM_network.dt <- rbindlist(postLocProb_SDM_network.list)[
  , model := 'SDM'][
  , prior := 'network']
saveRDS(postLocProb_RSS_uniform.dt, file.path(path_intermediate, 'postLocProb_RSS_uniform.dt.rds'))
saveRDS(postLocProb_RSS_network.dt, file.path(path_intermediate, 'postLocProb_RSS_network.dt.rds'))
saveRDS(postLocProb_SDM_uniform.dt, file.path(path_intermediate, 'postLocProb_SDM_uniform.dt.rds'))
saveRDS(postLocProb_SDM_network.dt, file.path(path_intermediate, 'postLocProb_SDM_network.dt.rds'))


postJointLocProb_RSS_uniform.dt <- rbindlist(postJointLocProb_RSS_uniform.list)[
  , model := 'RSS'][
  , prior := 'uniform']
postJointLocProb_RSS_network.dt <- rbindlist(postJointLocProb_RSS_network.list)[
  , model := 'RSS'][
  , prior := 'network']
postJointLocProb_SDM_uniform.dt <- rbindlist(postJointLocProb_SDM_uniform.list)[
  , model := 'SDM'][
  , prior := 'uniform']
postJointLocProb_SDM_network.dt <- rbindlist(postJointLocProb_SDM_network.list)[
  , model := 'SDM'][
  , prior := 'network']
saveRDS(postJointLocProb_RSS_uniform.dt, file.path(path_intermediate, 'postJointLocProb_RSS_uniform.dt.rds'))
saveRDS(postJointLocProb_RSS_network.dt, file.path(path_intermediate, 'postJointLocProb_RSS_network.dt.rds'))
saveRDS(postJointLocProb_SDM_uniform.dt, file.path(path_intermediate, 'postJointLocProb_SDM_uniform.dt.rds'))
saveRDS(postJointLocProb_SDM_network.dt, file.path(path_intermediate, 'postJointLocProb_SDM_network.dt.rds'))

# rm("postLocProb_RSS_uniform.dt")
# rm("postLocProb_RSS_network.dt")
# rm("postLocProb_SDM_uniform.dt")
# rm("postLocProb_SDM_network.dt")
# rm("postJointLocProb_RSS_uniform.dt")
# rm("postJointLocProb_RSS_network.dt")
# gc()
# saveRDS(postJointLocProb_SDM_uniform.list, file.path(path_intermediate, 'postJointLocProb_SDM_uniform.list.rds'))
# saveRDS(postJointLocProb_SDM_network.list, file.path(path_intermediate, 'postJointLocProb_SDM_network.list.rds'))
# 
# postJointLocProb_SDM_uniform.list <- readRDS(file.path(path_intermediate, 'postJointLocProb_SDM_uniform.list.rds'))
# postJointLocProb_SDM_uniform.dt <- rbindlist(postJointLocProb_SDM_uniform.list)
# saveRDS(postJointLocProb_SDM_uniform.dt, file.path(path_intermediate, 'postJointLocProb_SDM_uniform.dt.rds'))
# 
# 
# postJointLocProb_SDM_network.list <- readRDS(file.path(path_intermediate, 'postJointLocProb_SDM_network.list.rds'))
# postJointLocProb_SDM_network.dt <- rbindlist(postJointLocProb_SDM_network.list)
# saveRDS(postJointLocProb_SDM_network.dt, file.path(path_intermediate, 'postJointLocProb_SDM_network.dt.rds'))
# 

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####              PLOT EXAMPLE POSTLOC PROBABILITIES (GIF)                  ####
devID <- deviceIDs[12]

#### * RSS uniform                                                          ####
postLocLayers_device.list <- vector(mode = 'list', length = length(times))
names(postLocLayers_device.list) <- as.character(times)
for (locTime in times){
  cat(paste0('Layer for time ', locTime, '...'))
  temp.raster <- createRaster(map.sp, gridParam)
  values(temp.raster) <- postLocProb_RSS_uniform.dt[
    device == devID][
    time == locTime][
    tileEquiv.dt, on = 'tile'][
    order(rasterCell), eventLoc]
  postLocLayers_device.list[[as.character(locTime)]] <- temp.raster
  cat(' ok.\n')
}
postLocLayers_device.brick <- brick(Reduce(stack, postLocLayers_device.list))
animation::saveGIF(
  raster::animate(postLocLayers_device.brick, n = 1, pause = 0.01, main = '',
                  zlim = c(0, max(postLocProb_RSS_uniform.dt[device == devID, eventLoc])), 
                  col = rev(heat.colors(10))), 
  movie.name = file.path(path_image, paste0('postLocLayers_RSS_uniform_device', devID, '.gif')))

#### * RSS network                                                          ####
postLocLayers_device.list <- vector(mode = 'list', length = length(times))
names(postLocLayers_device.list) <- as.character(times)
for (locTime in times){
  cat(paste0('Layer for time ', locTime, '...'))
  temp.raster <- createRaster(map.sp, gridParam)
  values(temp.raster) <- postLocProb_RSS_network.dt[
    device == devID][
      time == locTime][
        tileEquiv.dt, on = 'tile'][
          order(rasterCell), eventLoc]
  postLocLayers_device.list[[as.character(locTime)]] <- temp.raster
  cat(' ok.\n')
}
postLocLayers_device.brick <- brick(Reduce(stack, postLocLayers_device.list))
animation::saveGIF(
  raster::animate(postLocLayers_device.brick, n = 1, pause = 0.01, main = '',
                  zlim = c(0, max(postLocProb_RSS_network.dt[device == devID, eventLoc])), 
                  col = rev(heat.colors(10))), 
  movie.name = file.path(path_image, paste0('postLocLayers_RSS_network_device', devID, '.gif')))

#### * SDM uniform                                                          ####
postLocLayers_device.list <- vector(mode = 'list', length = length(times))
names(postLocLayers_device.list) <- as.character(times)
for (locTime in times){
  cat(paste0('Layer for time ', locTime, '...'))
  temp.raster <- createRaster(map.sp, gridParam)
  values(temp.raster) <- postLocProb_SDM_uniform.dt[
    device == devID][
      time == locTime][
        tileEquiv.dt, on = 'tile'][
          order(rasterCell), eventLoc]
  postLocLayers_device.list[[as.character(locTime)]] <- temp.raster
  cat(' ok.\n')
}
postLocLayers_device.brick <- brick(Reduce(stack, postLocLayers_device.list))
animation::saveGIF(
  raster::animate(postLocLayers_device.brick, n = 1, pause = 0.01, main = '',
                  zlim = c(0, max(postLocProb_SDM_uniform.dt[device == devID, eventLoc])), 
                  col = rev(heat.colors(10))), 
  movie.name = file.path(path_image, paste0('postLocLayers_SDM_uniform_device', devID, '.gif')))

#### * SDM network                                                          ####
postLocLayers_device.list <- vector(mode = 'list', length = length(times))
names(postLocLayers_device.list) <- as.character(times)
for (locTime in times){
  cat(paste0('Layer for time ', locTime, '...'))
  temp.raster <- createRaster(map.sp, gridParam)
  values(temp.raster) <- postLocProb_SDM_network.dt[
    device == devID][
      time == locTime][
        tileEquiv.dt, on = 'tile'][
          order(rasterCell), eventLoc]
  postLocLayers_device.list[[as.character(locTime)]] <- temp.raster
  cat(' ok.\n')
}
postLocLayers_device.brick <- brick(Reduce(stack, postLocLayers_device.list))
animation::saveGIF(
  raster::animate(postLocLayers_device.brick, n = 1, pause = 0.01, main = '',
                  zlim = c(0, max(postLocProb_SDM_network.dt[device == devID, eventLoc])), 
                  col = rev(heat.colors(10))), 
  movie.name = file.path(path_image, paste0('postLocLayers_SDM_network_device', devID, '.gif')))



####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####              PLOT EXAMPLE POSTJOINTLOC PROBABILITIES (GIF)             ####
postJointLocLayers_device.list <- vector(mode = 'list', length = length(times[-1]))
names(postJointLocLayers_device.list) <- as.character(times[-1])
for (locTime in times[-1]){
  cat(paste0('Layer for time ', locTime, '...'))
  
  temp.DT <- postJointLocProb_RSS_uniform.dt[
    device == devID][
    time_to == locTime]
  temp.DT <- merge(temp.DT, tileEquiv.dt, by.x = 'tile_from', by.y = 'tile')
  setnames(temp.DT, 'rasterCell', 'rasterCell_from')
  temp.DT <- merge(temp.DT, tileEquiv.dt, by.x = 'tile_to', by.y = 'tile')
  setnames(temp.DT, 'rasterCell', 'rasterCell_to')
  temp.mat <- sparseMatrix(
    i = as.integer(temp.DT$rasterCell_from), 
    j = as.integer(temp.DT$rasterCell_to), 
    x = temp.DT$eventLoc, 
    dims = c(nTiles, nTiles))
  temp.raster <- raster(as.matrix(temp.mat))
  postJointLocLayers_device.list[[as.character(locTime)]] <- temp.raster
  cat(' ok.\n')
}
postJointLocLayers_device.brick <- brick(Reduce(stack, postJointLocLayers_device.list))
animation::saveGIF(
  raster::animate(postJointLocLayers_device.brick, n = 1, pause = 0.01, main = '',
                  zlim = c(0, max(postJointLocProb_RSS_uniform.dt[device == devID, eventLoc])), 
                  col = rev(heat.colors(10))), 
  movie.name = file.path(path_image, paste0('postJointLocLayers_device', devID, '.gif')))


