####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                           SET PATHS                                    ####
path_root         <- 'N:/UDMTD/UDTMDCOM/Cursos.Seminarios/BIGSURV20/edj-paper/supplMaterial'
path_source       <- file.path(path_root, 'source')
path_input        <- file.path(path_root, 'simulatorInputData')
path_output       <- file.path(path_root, 'simulatorOutputData')
path_intermediate <- file.path(path_root, 'intermediateData')
path_image        <- file.path(path_root, 'image')

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                  LOAD PACKAGES AND FUNCTIONS                           ####
library(xml2)
library(data.table)
library(destim)
library(stringr)
library(ggplot2)
library(ggimage)
library(scales)
library(raster)
library(tidyr)                # transform xml documents to tables (tibbles)
library(rgeos)                # spatial data
library(raster)               # spatial data
library(sf)                   # spatial data
library(gganimate)
library(magick)
library(latex2exp)

# Function to compute the radius of dispersion of a postloc distribution
source(file.path(path_source, 'rd.R'))

# Function to compute the center of probability of a postloc distribution
source(file.path(path_source, 'cp.R'))

# Function to compute the radius of gyration of a trajectory
source(file.path(path_source, 'rg.R'))

# Function to compute the center of mass of a trajectory
source(file.path(path_source, 'cm.R'))


# Function to create rasters
source(file.path(path_source, 'createRaster.R'))

# Function to construct the transition matrix from an HMM object
source(file.path(path_source, 'build_transMatrix.R'))


# Function to compute event location (likelihoods)
source(file.path(path_source, 'computeEventLoc.R'))

# Function to fit an HMM and compute posterior location probabilities
source(file.path(path_source, 'compute_HMM.R'))

# Function to transform de output of compute_HMM
source(file.path(path_source, 'transform_postLoc.R'))


####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                      READ RDS DATA FILES                               ####
RSS.dt <- readRDS(file.path(path_intermediate, 'RSS.dt.rds'))

gridParam      <- readRDS(file.path(path_intermediate, 'gridParam.rds'))
ncols <- gridParam[['No Tiles Y']] # está ya en los parámetros: ncol_grid
nrows <- gridParam[['No Tiles X']] # está ya en los parámetros: nrow_grid
nTiles <- ncols * nrows
tileSize <- mean(c(gridParam$`X Tile Dim`, gridParam$`Y Tile Dim`))

simulationParam <- readRDS(file.path(path_intermediate, 'simulation.xml.rds'))
initialTime <- as.numeric(simulationParam$simulation$start_time[[1]])
finalTime   <- as.numeric(simulationParam$simulation$end_time[[1]])
timeIncrem  <- as.numeric(simulationParam$simulation$time_increment[[1]])

events.dt <- readRDS(file.path(path_intermediate, 'events.dt.rds'))
setnames(events.dt, c('deviceID', 'tile', 'rasterCell'), c('device', 'trueTile', 'trueRasterCell'))

tileEquiv.dt <- readRDS(file.path(path_intermediate, 'tileEquiv.dt.rds'))

postLocProb_RSS_uniform.dt <- readRDS(file.path(path_intermediate, 'postLocProb_RSS_uniform.dt.rds'))
postLocProb_RSS_network.dt <- readRDS(file.path(path_intermediate, 'postLocProb_RSS_network.dt.rds'))
postLocProb_SDM_uniform.dt <- readRDS(file.path(path_intermediate, 'postLocProb_SDM_uniform.dt.rds'))
postLocProb_SDM_network.dt <- readRDS(file.path(path_intermediate, 'postLocProb_SDM_network.dt.rds'))

centroidCoord.dt <- readRDS(file.path(path_intermediate, 'centroidCoord.dt.rds'))

truePositions.dt <- readRDS(file.path(path_intermediate, 'positionsConnections.dt.rds'))
setnames(truePositions.dt, 'deviceID', 'device')

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####      CENTER OF PROBABILITY AND ROOT MEAN SQUARED DISPERSION            ####

#### * RSS uniform                                                          ####
postLocProb_RSS_uniform.dt <- merge(
  postLocProb_RSS_uniform.dt, centroidCoord.dt, by = 'tile')

#### ** b                                                                   ####
cp_truePos_RSS_uniform.dt <- postLocProb_RSS_uniform.dt[
  , as.list(cp(x = centroidCoord_x, y = centroidCoord_y, w = eventLoc)), by = c('device', 'time')][
  truePositions.dt[, .(time, device, x, y)], on = c('device', 'time')][
  , cp_tp := sqrt( (cp_x - x)**2 + (cp_y - y)**2)][
  , .(device, time, cp_tp)][
  , model := 'RSS_uniform'][
  order(device, time)]

#### ** rmsd                                                                ####
rmsd_RSS_uniform.dt <- postLocProb_RSS_uniform.dt[
  , list(rd = rd(x = centroidCoord_x, y = centroidCoord_y, w = eventLoc)), by = c('device', 'time')][
  , model := 'RSS_uniform']


#### * RSS network                                                          ####
postLocProb_RSS_network.dt <- merge(
  postLocProb_RSS_network.dt, centroidCoord.dt, by = 'tile')

#### ** b                                                                   ####
cp_truePos_RSS_network.dt <- postLocProb_RSS_network.dt[
  , as.list(cp(x = centroidCoord_x, y = centroidCoord_y, w = eventLoc)), by = c('device', 'time')][
  truePositions.dt[, .(time, device, x, y)], on = c('device', 'time')][
  , cp_tp := sqrt( (cp_x - x)**2 + (cp_y - y)**2)][
  , .(device, time, cp_tp)][
  , model := 'RSS_network'][
  order(device, time)]

#### ** rmsd                                                                ####
rmsd_RSS_network.dt <- postLocProb_RSS_network.dt[
  , list(rd = rd(x = centroidCoord_x, y = centroidCoord_y, w = eventLoc)), by = c('device', 'time')][
  , model := 'RSS_network']


#### * SDM uniform                                                          ####
postLocProb_SDM_uniform.dt <- merge(
  postLocProb_SDM_uniform.dt, centroidCoord.dt, by = 'tile')

#### ** b                                                                   ####
cp_truePos_SDM_uniform.dt <- postLocProb_SDM_uniform.dt[
  , as.list(cp(x = centroidCoord_x, y = centroidCoord_y, w = eventLoc)), by = c('device', 'time')][
  truePositions.dt[, .(time, device, x, y)], on = c('device', 'time')][
  , cp_tp := sqrt( (cp_x - x)**2 + (cp_y - y)**2)][
  , .(device, time, cp_tp)][
  , model := 'SDM_uniform'][
  order(device, time)]

#### ** rmsd                                                                ####
rmsd_SDM_uniform.dt <- postLocProb_SDM_uniform.dt[
  , list(rd = rd(x = centroidCoord_x, y = centroidCoord_y, w = eventLoc)), by = c('device', 'time')][
  , model := 'SDM_uniform']


#### * SDM network                                                          ####
postLocProb_SDM_network.dt <- merge(
  postLocProb_SDM_network.dt, centroidCoord.dt, by = 'tile')

#### ** b                                                                   ####
cp_truePos_SDM_network.dt <- postLocProb_SDM_network.dt[
  , as.list(cp(x = centroidCoord_x, y = centroidCoord_y, w = eventLoc)), by = c('device', 'time')][
  truePositions.dt[, .(time, device, x, y)], on = c('device', 'time')][
  , cp_tp := sqrt( (cp_x - x)**2 + (cp_y - y)**2)][
  , .(device, time, cp_tp)][
  , model := 'SDM_network'][
  order(device, time)]

#### ** rmsd                                                                ####
rmsd_SDM_network.dt <- postLocProb_SDM_network.dt[
  , list(rd = rd(x = centroidCoord_x, y = centroidCoord_y, w = eventLoc)), by = c('device', 'time')][
  , model := 'SDM_network']


cp_truePos.dt <- rbindlist(list(
  cp_truePos_RSS_uniform.dt,
  cp_truePos_RSS_network.dt,
  cp_truePos_SDM_uniform.dt,
  cp_truePos_SDM_network.dt))

rmsd.dt <- rbindlist(list(
  rmsd_RSS_uniform.dt,
  rmsd_RSS_network.dt,
  rmsd_SDM_uniform.dt,
  rmsd_SDM_network.dt))

msd.dt <- merge(cp_truePos.dt, rmsd.dt, by = c('device', 'time', 'model'))
msd.dt[
  , msd := cp_tp**2 + rd**2]
saveRDS(msd.dt, file.path(path_intermediate, 'msd.dt.rds'))

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####              PLOT DISTRIBUTIONS of b, rmsd AND msd                     ####
ggplot(msd.dt, aes(x = model, y = cp_tp / tileSize)) +
  geom_boxplot(aes(fill = model)) +
  theme_bw() +
  labs(x = '', y = TeX('$b_{dt}$ (no. tiles)\n'), title = '') +
  theme(plot.title = element_text(hjust = 0.5, size = 18),
        axis.text.x = element_text(size = 16), 
        axis.title.y = element_text(size = 16), axis.text.y = element_text(size = 12),
        legend.title = element_text(size = 14), legend.text = element_text(size = 12),
        legend.position = 'none')

ggplot(msd.dt, aes(x = time, y = cp_tp / tileSize, group = time)) +
  geom_boxplot(aes(fill = model)) +
  facet_grid(model ~ . ) +
  theme_bw() +
  labs(x = '\nTime (s)', y = TeX('$b_{dt}$ (no. tiles)\n'), title = '') +
  theme(plot.title = element_text(hjust = 0.5, size = 18),
        axis.title.x = element_text(size = 16),axis.text.x = element_text(size = 16), 
        axis.title.y = element_text(size = 16), axis.text.y = element_text(size = 12),
        legend.title = element_text(size = 14), legend.text = element_text(size = 12),
        legend.position = 'none')

ggplot(msd.dt, aes(x = device, y = cp_tp / tileSize, group = device)) +
  geom_boxplot(aes(fill = model)) +
  facet_grid(model ~ . ) +
  theme_bw() +
  labs(x = '\nDevice', y = TeX('$b_{dt}$ (no. tiles)\n'), title = '') +
  theme(plot.title = element_text(hjust = 0.5, size = 18),
        axis.title.x = element_text(size = 16),axis.text.x = element_text(size = 6, angle = 90), 
        axis.title.y = element_text(size = 16), axis.text.y = element_text(size = 12),
        legend.title = element_text(size = 14), legend.text = element_text(size = 12),
        legend.position = 'none')


ggplot(msd.dt, aes(x = model, y = rd / tileSize)) +
  geom_boxplot(aes(fill = model)) +
  theme_bw() +
  labs(x = '', y = 'rmsd (no. tiles)\n', title = '') +
  theme(plot.title = element_text(hjust = 0.5, size = 18),
        axis.text.x = element_text(size = 16), 
        axis.title.y = element_text(size = 16), axis.text.y = element_text(size = 12),
        legend.title = element_text(size = 14), legend.text = element_text(size = 12),
        legend.position = 'none')


ggplot(msd.dt, aes(x = time, y = rd / tileSize, group = time)) +
  geom_boxplot(aes(fill = model)) +
  facet_grid(model ~ . ) +
  theme_bw() +
  labs(x = '\nTime (s)', y = 'rmsd (no. tiles)\n', title = '') +
  theme(plot.title = element_text(hjust = 0.5, size = 18),
        axis.title.x = element_text(size = 16),axis.text.x = element_text(size = 16), 
        axis.title.y = element_text(size = 16), axis.text.y = element_text(size = 12),
        legend.title = element_text(size = 14), legend.text = element_text(size = 12),
        legend.position = 'none')

ggplot(msd.dt, aes(x = device, y = rd / tileSize, group = device)) +
  geom_boxplot(aes(fill = model)) +
  facet_grid(model ~ . ) +
  theme_bw() +
  labs(x = '\nDevice', y = 'rmsd (no. tiles)\n', title = '') +
  theme(plot.title = element_text(hjust = 0.5, size = 18),
        axis.title.x = element_text(size = 16),axis.text.x = element_text(size = 6, angle = 90), 
        axis.title.y = element_text(size = 16), axis.text.y = element_text(size = 12),
        legend.title = element_text(size = 14), legend.text = element_text(size = 12),
        legend.position = 'none')



ggplot(msd.dt, aes(x = model, y = msd / tileSize**2)) +
  geom_boxplot(aes(fill = model)) +
  theme_bw() +
  labs(x = '', y = 'msd (no. tiles)\n', title = '') +
  theme(plot.title = element_text(hjust = 0.5, size = 18),
        axis.text.x = element_text(size = 16), 
        axis.title.y = element_text(size = 16), axis.text.y = element_text(size = 12),
        legend.title = element_text(size = 14), legend.text = element_text(size = 12),
        legend.position = 'none')


ggplot(msd.dt, aes(x = time, y = msd / tileSize**2, group = time)) +
  geom_boxplot(aes(fill = model)) +
  facet_grid(model ~ . ) +
  theme_bw() +
  labs(x = '\nTime (s)', y = 'msd (no. tiles)\n', title = '') +
  theme(plot.title = element_text(hjust = 0.5, size = 18),
        axis.title.x = element_text(size = 16),axis.text.x = element_text(size = 16), 
        axis.title.y = element_text(size = 16), axis.text.y = element_text(size = 12),
        legend.title = element_text(size = 14), legend.text = element_text(size = 12),
        legend.position = 'none')


ggplot(msd.dt, aes(x = device, y = msd / tileSize**2, group = device)) +
  geom_boxplot(aes(fill = model)) +
  facet_grid(model ~ . ) +
  theme_bw() +
  labs(x = '\nDevice', y = TeX('msd (no. tiles$^{2}$)\n'), title = '') +
  theme(plot.title = element_text(hjust = 0.5, size = 18),
        axis.title.x = element_text(size = 16),axis.text.x = element_text(size = 6, angle = 90), 
        axis.title.y = element_text(size = 16), axis.text.y = element_text(size = 12),
        legend.title = element_text(size = 14), legend.text = element_text(size = 12),
        legend.position = 'none')


####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####            PLOT DEVICE TIME SERIES OF of b, rmsd AND msd               ####
devID <- sort(devIDs)[5]
ggplot(msd.dt[device == devID], aes(x = time, y = cp_tp / tileSize, group = time)) +
  geom_point(aes(color = model)) +
  facet_grid(model ~ . ) +
  scale_y_continuous(limits = c(0, NA)) +
  theme_bw() +
  labs(x = '\nTime (s)', y = TeX('$b_{dt}$ (no. tiles)\n'), title = paste0('Device ', devID)) +
  theme(plot.title = element_text(hjust = 0.5, size = 18),
        axis.title.x = element_text(size = 16),axis.text.x = element_text(size = 16), 
        axis.title.y = element_text(size = 16), axis.text.y = element_text(size = 12),
        legend.title = element_text(size = 14), legend.text = element_text(size = 12),
        legend.position = 'none')

ggplot(msd.dt[device == devID], aes(x = time, y = rd / tileSize, group = time)) +
  geom_point(aes(color = model)) +
  facet_grid(model ~ . ) +
  scale_y_continuous(limits = c(0, NA)) +
  theme_bw() +
  labs(x = '\nTime (s)', y = TeX('$b_{dt}$ (no. tiles)\n'), title = paste0('Device ', devID)) +
  theme(plot.title = element_text(hjust = 0.5, size = 18),
        axis.title.x = element_text(size = 16),axis.text.x = element_text(size = 16), 
        axis.title.y = element_text(size = 16), axis.text.y = element_text(size = 12),
        legend.title = element_text(size = 14), legend.text = element_text(size = 12),
        legend.position = 'none')

ggplot(msd.dt[device == devID], aes(x = time, y = msd / tileSize**2, group = time)) +
  geom_point(aes(color = model)) +
  facet_grid(model ~ . ) +
  scale_y_continuous(limits = c(0, NA)) +
  theme_bw() +
  labs(x = '\nTime (s)', y = TeX('$b_{dt}$ (no. tiles)\n'), title = paste0('Device ', devID)) +
  theme(plot.title = element_text(hjust = 0.5, size = 18),
        axis.title.x = element_text(size = 16),axis.text.x = element_text(size = 16), 
        axis.title.y = element_text(size = 16), axis.text.y = element_text(size = 12),
        legend.title = element_text(size = 14), legend.text = element_text(size = 12),
        legend.position = 'none')
